/*
 * ztstatd.c
 * $Id$
 *
 * Used for remote monitoring of T1/E1 spans
 *
 * Derived from zttool.c
 * All other code is Copyright (c) 2005 by Tilghman Lesher
 * Released under the terms of the GPL (or other license if licensed by Digium)
 *
 */ 

#include <stdio.h> 
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <errno.h>
#include <linux/zaptel.h>
#include <syslog.h>
#include <sys/signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

int ctl, sock, conn;
int port = 5039;

void ignore_signal(int signal)
{
	close(ctl);
	shutdown(conn, 2);
	return;
}

void shutdown_signal(int signal)
{
	close(ctl);
	shutdown(conn, 2);
	shutdown(sock, 2);
	syslog(LOG_NOTICE, "Shutting down");
	closelog();
	exit(0);
}

int main(int argc, char **argv)
{
	int res, one = 1, i;
	struct sockaddr_in addr = { 0 }, addr2 = { 0 };
	int addrlen = sizeof(addr2);
	char alarms[50] = "";
	ZT_SPANINFO info;

	signal(SIGPIPE, ignore_signal);
	signal(SIGHUP, shutdown_signal);

	openlog("ztstatd", LOG_PID, LOG_DAEMON);
	if (fork() > 0) {
		exit(0);
	}

	fclose(stdin);
	fclose(stdout);
	fclose(stderr);

	sock = socket(PF_INET, SOCK_STREAM, 0);
	setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &one, sizeof(one));
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);
	bind(sock, (struct sockaddr *)&addr, sizeof(addr));
	listen(sock, SOMAXCONN);

	while ((conn = accept(sock, (struct sockaddr *)&addr2, &addrlen))) {
		FILE *CONN = fdopen(conn, "r+");
		syslog(LOG_NOTICE, "Accepted connection from %s, port %d", inet_ntoa(addr2.sin_addr), ntohs(addr2.sin_port));

		ctl = open("/dev/zap/ctl", O_RDWR);
		if (ctl < 0) {
			syslog(LOG_CRIT, "Unable to open /dev/zap/ctl: %s", strerror(errno));
			fprintf(CONN, "0:ERROR\r\n");
			fflush(CONN);
			shutdown(conn, 2);
			continue;
		}

		for (i = 1; i <= ZT_MAX_SPANS; i++) {
			info.spanno = i;

			res = ioctl(ctl, ZT_SPANSTAT, &info);
			if (res) {
				/* No such span */
				break;
			}

			if (info.alarms > 0) {
				alarms[0] = '\0';
				if (info.alarms & ZT_ALARM_BLUE)
					strcat(alarms,"BLU/");
				if (info.alarms & ZT_ALARM_YELLOW)
					strcat(alarms, "YEL/");
				if (info.alarms & ZT_ALARM_RED)
					strcat(alarms, "RED/");
				if (info.alarms & ZT_ALARM_LOOPBACK)
					strcat(alarms,"LB/");
				if (info.alarms & ZT_ALARM_RECOVER)
					strcat(alarms,"REC/");
				if (info.alarms & ZT_ALARM_NOTOPEN)
					strcat(alarms, "NOP/");
				if (!strlen(alarms))
					strcat(alarms, "UUU/");
				if (strlen(alarms)) {
					/* Strip trailing / */
					alarms[strlen(alarms)-1]='\0';
				}
			} else {
				if (info.numchans)
					strcpy(alarms, "OK");
				else
					strcpy(alarms, "UNCONFIGURED");
			}
			fprintf(CONN, "%d:%s\r\n", i, alarms);
		}
		fflush(CONN);
		shutdown(conn, 2);
		close(ctl);
	}
	return 0;
}


